@@ -22,14 +22,16 @@
 #include "state_space_poisson_model_manager.h"
 #include "state_space_regression_model_manager.h"
 #include "state_space_student_model_manager.h"
+// #include "multivariate_gaussian_model_manager.h"
+
 #include "utils.h"
+#include "create_state_model.h"
+#include "create_shared_state_model.h"
 
 #include "r_interface/boom_r_tools.hpp"
-#include "r_interface/create_state_model.hpp"
 #include "r_interface/list_io.hpp"
 
 #include "Models/StateSpace/Filters/KalmanTools.hpp"
-#include "Models/StateSpace/StateModels/DynamicRegressionStateModel.hpp"
 
 #include "cpputil/report_error.hpp"
 #include "distributions.hpp"
@@ -37,60 +39,96 @@
 namespace BOOM {
   namespace bsts {
 
+    TimestampInfo::TimestampInfo(SEXP r_data_list) {
+      Unpack(r_data_list);
+    }
+    
+    void TimestampInfo::Unpack(SEXP r_data_list) {
+      SEXP r_timestamp_info = getListElement(r_data_list, "timestamp.info");
+      trivial_ = Rf_asLogical(getListElement(
+          r_timestamp_info, "timestamps.are.trivial"));
+      number_of_time_points_ = Rf_asInteger(getListElement(
+          r_timestamp_info, "number.of.time.points"));
+      if (!trivial_) {
+        timestamp_mapping_ = ToIntVector(getListElement(
+            r_timestamp_info, "timestamp.mapping"));
+      }
+    }
+
+    // Args:
+    //   r_prediction_data: A list containing an object named 'timestamps',
+    //     which is a list containing the following objects.  
+    //     - timestamp.mapping: A vector of integers indicating the timestamp to
+    //         which each observation belongs.
+    //
+    // Effects:
+    //   The forecast_timestamps_ element in the TimestampInfo object gets
+    //   populated.
+    void TimestampInfo::UnpackForecastTimestamps(SEXP r_prediction_data) {
+      SEXP r_forecast_timestamps = getListElement(
+          r_prediction_data, "timestamps");
+      if (!Rf_isNull(r_forecast_timestamps)) {
+        forecast_timestamps_ = ToIntVector(getListElement(
+            r_forecast_timestamps, "timestamp.mapping"));
+        for (int i = 1; i < forecast_timestamps_.size(); ++i) {
+          if (forecast_timestamps_[i] < forecast_timestamps_[i - 1]) {
+            report_error("Time stamps for multiplex predictions must be "
+                         "in increasing order.");
+          }
+        }
+      }
+    }
+
+    //==========================================================================
+    
     // The model manager will be thread safe as long as it is created from the
     // home thread.
     ModelManager::ModelManager()
-        : rng_(seed_rng(GlobalRng::rng)),
-          timestamps_are_trivial_(true),
-          number_of_time_points_(-1) {}
+        : rng_(seed_rng(GlobalRng::rng)) {}
 
-    ModelManager * ModelManager::Create(SEXP r_bsts_object) {  // NOLINT
-      std::string family = ToString(getListElement(
-          r_bsts_object, "family"));
-      bool regression = !Rf_isNull(getListElement(
-          r_bsts_object, "predictors"));
+    ScalarModelManager * ScalarModelManager::Create(SEXP r_bsts_object) {  
+      std::string family = ToString(getListElement(r_bsts_object, "family"));
+      bool regression = !Rf_isNull(getListElement(r_bsts_object, "predictors"));
       int xdim = 0;
       if (regression) {
         xdim = Rf_ncols(getListElement(r_bsts_object, "predictors"));
       }
-      return ModelManager::Create(family, xdim);
+      return ScalarModelManager::Create(family, xdim);
     }
 
-    ModelManager *ModelManager::Create(const std::string &family_name,  
-                                       int xdim) {
-      ModelManager *ans = nullptr;
+    ScalarModelManager *ScalarModelManager::Create(
+        const std::string &family_name, int xdim) {
       if (family_name == "gaussian") {
         if (xdim > 0) {
           StateSpaceRegressionModelManager *manager =
               new StateSpaceRegressionModelManager;
           manager->SetPredictorDimension(xdim);
-          ans = manager;
+          return manager;
         } else {
-          ans = new StateSpaceModelManager;
+          return new StateSpaceModelManager;
         }
       } else if (family_name == "logit") {
         StateSpaceLogitModelManager *manager =
             new StateSpaceLogitModelManager;
         manager->SetPredictorDimension(xdim);
-        ans = manager;
+        return manager;
       } else if (family_name == "poisson") {
         StateSpacePoissonModelManager *manager =
             new StateSpacePoissonModelManager;
         manager->SetPredictorDimension(xdim);
-        ans = manager;
+        return manager;
       } else if (family_name == "student") {
         StateSpaceStudentModelManager *manager
             = new StateSpaceStudentModelManager;
         manager->SetPredictorDimension(xdim);
-        ans = manager;
+        return manager;
       } else {
         std::ostringstream err;
         err << "Unrecognized family name: " << family_name
             << " in ModelManager::Create.";
         report_error(err.str());
-        return nullptr;
       }
-      return ans;
+      return nullptr;
     }
 
     class FullStateCallback : public MatrixIoCallback {
@@ -103,21 +141,20 @@ namespace BOOM {
       StateSpaceModelBase *model_;
     };
     
-    ScalarStateSpaceModelBase * ModelManager::CreateModel(  // NOLINT
+    ScalarStateSpaceModelBase * ScalarModelManager::CreateModel(  
         SEXP r_data_list,
         SEXP r_state_specification,
         SEXP r_prior,
         SEXP r_options,
-        Vector *final_state,
         RListIoManager *io_manager) {
-      ScalarStateSpaceModelBase *model = CreateObservationModel(
+      ScalarStateSpaceModelBase *model = CreateBareModel(
           r_data_list,
           r_prior,
           r_options,
           io_manager);
-      RInterface::StateModelFactory state_model_factory(io_manager, model);
-      state_model_factory.AddState(r_state_specification);
-      state_model_factory.SaveFinalState(final_state);
+      StateModelFactory state_model_factory(io_manager);
+      state_model_factory.AddState(model, r_state_specification, "");
+      state_model_factory.SaveFinalState(model, &final_state());
 
       // The predict method does not set BstsOptions, so allow NULL here to
       // signal that options have not been set.
@@ -127,7 +164,7 @@ namespace BOOM {
         if (save_state_contribution) {
           io_manager->add_list_element(
               new NativeMatrixListElement(
-                  new GeneralStateContributionCallback(model),
+                  new ScalarStateContributionCallback(model),
                   "state.contributions",
                   nullptr));
         }
@@ -157,12 +194,39 @@ namespace BOOM {
       return model;
     }
 
-    Matrix ModelManager::Forecast(SEXP r_bsts_object,
-                                  SEXP r_prediction_data,
-                                  SEXP r_burn,
-                                  SEXP r_observed_data) {
+    // Primary implementation of predict.bsts.  Child classes will carry out
+    // some of the details, but most of the prediction logic is here.
+    //
+    // Args:
+    //   r_bsts_object:  The bsts model object for which a prediction is deisred.
+    //   r_prediction_data: An R list containing any additional data needed to
+    //     make the prediction.  For simple state space models this is just an
+    //     integer giving the time horizon over which to predict.  For models
+    //     containing a regression component it contains the future values of
+    //     the X's.  For binomial (or Poisson) models it contains a sequence of
+    //     future trial counts (or exposures).
+    //   r_burn: An integer giving the number of burn-in iterations to discard.
+    //     Negative numbers will be treated as zero.  Numbers greater than the
+    //     number of MCMC iterations will raise an error.
+    //   r_observed_data: An R list containing the observed data on which to
+    //     base the prediction.  In typical cases this should be R_NilValue (R's
+    //     NULL) signaling that the observed data should be taken from
+    //     r_bsts_object.  However, if additional data have been observed since
+    //     the model was trained, or if the model is being used to predict data
+    //     that were part of the training set, or some other application other
+    //     than making predictions starting from one time period after the
+    //     training data ends, then one can use this argument to pass the
+    //     "training data" on which the predictions should be based.  If this
+    //     argument is used, then the Kalman filter will be run over the
+    //     supplied data, which is expensive.  If this argument is left as
+    //     R_NilValue (NULL) then the "final.state" element of r_bsts_object
+    //     will be used as the basis for future predictions, which is a
+    //     computational savings over filtering from scratch.
+    Matrix ScalarModelManager::Forecast(SEXP r_bsts_object,
+                                        SEXP r_prediction_data,
+                                        SEXP r_burn,
+                                        SEXP r_observed_data) {
       RListIoManager io_manager;
-      Vector final_state;
       SEXP r_state_specfication = getListElement(
           r_bsts_object, "state.specification");
       ScalarStateSpaceModelBase *model = CreateModel(
@@ -170,7 +234,6 @@ namespace BOOM {
           r_state_specfication,
           R_NilValue,
           R_NilValue,
-          &final_state,
           &io_manager);
       bool refilter;
       if (Rf_isNull(r_observed_data)) {
@@ -182,6 +245,7 @@ namespace BOOM {
       }
       int niter = Rf_asInteger(getListElement(r_bsts_object, "niter"));
       int burn = std::max<int>(0, Rf_asInteger(r_burn));
+      if (burn < 0) burn = 0;
       io_manager.prepare_to_stream(r_bsts_object);
       io_manager.advance(burn);
       int iterations_after_burnin = niter - burn;
@@ -190,22 +254,25 @@ namespace BOOM {
         report_error("Forecast called with NULL prediction data.");
       }
       int forecast_horizon = UnpackForecastData(r_prediction_data);
-      UnpackDynamicRegressionForecastData(
-          model, r_state_specfication, r_prediction_data);
-
+      int max_time = model->time_dimension() + forecast_horizon;
+      for (int s = 0; s < model->number_of_state_models(); ++s) {
+        model->state_model(s)->observe_time_dimension(max_time);
+      }
       Matrix ans(iterations_after_burnin, forecast_horizon);
       for (int i = 0; i < iterations_after_burnin; ++i) {
         io_manager.stream();
+        Vector final_state = this->final_state();
         if (refilter) {
-          model->light_kalman_filter();
-          ScalarKalmanStorage storage = model->final_kalman_storage();
-          Vector state_mean = storage.a;
-          SpdMatrix state_variance = storage.P;
+          model->kalman_filter();
+          const Kalman::ScalarMarginalDistribution &marg(
+              model->get_filter().back());
+          Vector state_mean = marg.state_mean();
+          SpdMatrix state_variance = marg.state_variance();
           make_contemporaneous(
               state_mean,
               state_variance,
-              storage.F,
-              storage.v,
+              marg.prediction_variance(),
+              marg.prediction_error(),
               model->observation_matrix(model->time_dimension()).dense());
           final_state = rmvn(state_mean, state_variance);
         }
@@ -214,57 +281,37 @@ namespace BOOM {
       return ans;
     }
 
-    void ModelManager::UnpackDynamicRegressionForecastData(
-        ScalarStateSpaceModelBase *model,
-        SEXP r_state_specification,
-        SEXP r_prediction_data) {
-      if (Rf_length(r_state_specification) < model->nstate()) {
-        std::ostringstream err;
-        err << "The number of state components in the model: ("
-            << model->nstate() << ") does not match the size of "
-            << "the state specification: ("
-            << Rf_length(r_state_specification)
-            << ") in UnpackDynamicRegressionForecastData.";
-        report_error(err.str());
-      }
-      for (int i = 0; i < model->nstate(); ++i) {
-        SEXP spec = VECTOR_ELT(r_state_specification, i);
-        if (Rf_inherits(spec, "DynamicRegression")) {
-          Matrix predictors = ToBoomMatrix(getListElement(
-              r_prediction_data, "dynamic.regression.predictors"));
-          Ptr<StateModel> state_model = model->state_model(i);
-          state_model.dcast<DynamicRegressionStateModel>()->add_forecast_data(
-              predictors);
-        }
-      }
-    }
+    //=========================================================================
+    // MultivariateModelManagerBase * MultivariateModelManagerBase::Create(
+    //     SEXP r_mbsts_object) {
+    //   std::string family = ToString(getListElement(r_mbsts_object, "family"));
+    //   int ydim = Rf_ncols(getListElement(
+    //       r_mbsts_object, "original.series", true));
+    //   bool regression = !Rf_isNull(getListElement(
+    //       r_mbsts_object, "predictors", true));
+    //   int xdim = 0;
+    //   if (regression) {
+    //     xdim = Rf_ncols(getListElement(r_mbsts_object, "predictors"));
+    //   }
+    //   return MultivariateModelManagerBase::Create(family, ydim, xdim);
+    // }
 
-    void ModelManager::UnpackTimestampInfo(SEXP r_data_list) {
-      SEXP r_timestamp_info = getListElement(r_data_list, "timestamp.info");
-      timestamps_are_trivial_ = Rf_asLogical(getListElement(
-          r_timestamp_info, "timestamps.are.trivial"));
-      number_of_time_points_ = Rf_asInteger(getListElement(
-          r_timestamp_info, "number.of.time.points"));
-      if (!timestamps_are_trivial_) {
-        timestamp_mapping_ = ToIntVector(getListElement(
-            r_timestamp_info, "timestamp.mapping"));
-      }
-    }
+    // //--------------------------------------------------------------------------    
+    // MultivariateModelManagerBase * MultivariateModelManagerBase::Create(
+    //     const std::string &family, int ydim, int xdim) {
 
-    void ModelManager::UnpackForecastTimestamps(SEXP r_prediction_data) {
-      SEXP r_forecast_timestamps = getListElement(
-          r_prediction_data, "timestamps");
-      if (!Rf_isNull(r_forecast_timestamps)) {
-        forecast_timestamps_ = ToIntVector(getListElement(
-            r_forecast_timestamps, "timestamp.mapping"));
-        for (int i = 1; i < forecast_timestamps_.size(); ++i) {
-          if (forecast_timestamps_[i] < forecast_timestamps_[i - 1]) {
-            report_error("Time stamps for multiplex predictions must be "
-                         "in increasing order.");
-          }
-        }
-      }
-    }
+    //   if (family == "gaussian") {
+    //     MultivariateGaussianModelManager *manager =
+    //         new MultivariateGaussianModelManager(ydim, xdim);
+    //     return manager;
+    //   } else {
+    //     report_error("For now, only Gaussian families are supported in the "
+    //                  "multivariate case.");
+    //   }
+    //   return nullptr;
+    // }
 
+    //--------------------------------------------------------------------------
+    
   }  // namespace bsts
 }  // namespace BOOM
