@@ -31,7 +31,7 @@ typedef StateSpaceStudentModelManager SSSMM;
 SSSMM::StateSpaceStudentModelManager()
     : predictor_dimension_(-1) {}
 
-StateSpaceStudentRegressionModel * SSSMM::CreateObservationModel(
+StateSpaceStudentRegressionModel * SSSMM::CreateBareModel(
       SEXP r_data_list,
       SEXP r_prior,
       SEXP r_options,
@@ -156,6 +156,7 @@ StateSpaceStudentRegressionModel * SSSMM::CreateObservationModel(
 HoldoutErrorSampler SSSMM::CreateHoldoutSampler(
     SEXP r_bsts_object,
     int cutpoint,
+    bool standardize,
     Matrix *prediction_error_output) {
   RListIoManager io_manager;
   Ptr<StateSpaceStudentRegressionModel> model =
@@ -164,7 +165,6 @@ HoldoutErrorSampler SSSMM::CreateHoldoutSampler(
           getListElement(r_bsts_object, "state.specification"),
           getListElement(r_bsts_object, "prior"),
           getListElement(r_bsts_object, "model.options"),
-          nullptr,
           &io_manager));
   AddDataFromBstsObject(r_bsts_object);
 
@@ -192,6 +192,7 @@ HoldoutErrorSampler SSSMM::CreateHoldoutSampler(
   return HoldoutErrorSampler(new StateSpaceStudentHoldoutErrorSampler(
       model, holdout_response, holdout_predictors,
       Rf_asInteger(getListElement(r_bsts_object, "niter")),
+      standardize,
       prediction_error_output));
 }
 
